/**
 * Das Handling für den Live-Channel. Das ist
 * der einzige Kanal, der nicht über das normale Channel.js
 * gemanaged wird, da dieser Kanal zu speziell ist.
 *
 * Das ganze fragt regelmäßig die Methode LiveEvent.get
 * im Webservice ab und reagiert auf Events, die von
 * dort geschickt werden. 
 *
 * Zum Beispiel:
 * * Werbung zeigen
 * * Neues Video abspielen mit URL
 * * Grafik-Overlay anzeigen
 */
var LiveEventChannel = function () {
  var self = {};
  var lastVersion = null;
  var eventData = null;
  var liveEventId = null;
  var bauchbindeTimer = null;
  var curtainPicture = null;
  var advertId = null;
  var currentClip = null;

  function refreshData() {
    var aspect = eventData["aspect_ratio"];
    if (aspect == null) {
      aspect = 16/9;
    }
    currentClip = {
      "aspect_ratio": aspect,
      "artist": eventData["title"],
      "title": eventData["subtitle"],
      "live_streaming": true,
      "clipInfo": {
        "clip_container" : {}
      }
    };
    ChannelView.currentClip = currentClip;

    if (eventData["artist_info_asset_id"]) {
      currentClip["asset_id"] = eventData["artist_info_asset_id"];
      currentClip["asset_type"] = "MusicVideo";
    }

    if (!Utils.isStringEmpty(eventData["show_logo"])) {
      currentClip["clipInfo"]["clip_container"] = {
        "display_values": "PICTURE",
        "picture": eventData["show_logo"]
      };
    }

    if (ChannelView.mode != "ADVERT") {
      self.updateClip();
    }

    clearInterval(bauchbindeTimer);
    var binterval = parseInt(eventData["insert_interval"], 10);
    if (isNaN(binterval) || binterval == null || binterval == 0) {
      binterval = 60;
    }
    bauchbindeTimer = setInterval(function() {
      if (curtainPicture == null) {
        if (ChannelView.mode != "ADVERT" && curtainPicture == null) {
          MessageQueue.sendMessage("show_bauchbinde");
        }
      }
    }, binterval * 1000);

    lastVersion = eventData["version"];
  }

  function loadData(liveId) {
    liveEventId = liveId;
    fetchData();
  }

  function fetchData() {
    params["id"] = liveEventId;
    Putpat.callApi("LiveEvent.get", params, function(data) {
      eventData = data["live_event"];

      if (lastVersion != eventData["version"]) {
        refreshData();
      }
      var interval = parseInt(eventData["polling_interval"], 10);
      if (isNaN(interval) || interval == null || interval == 0) {
        interval = 5;
      }
      setTimeout(function() {
        fetchData();
      }, interval * 1000);
    });
  }

  self.updateClip = function() {
    ChannelView.resetCurrentClip();
    ChannelView.currentClip = currentClip;

    var newUrl = eventData["streaming_urls"]["http"]["high"];
    if (Device.name == "Samsung") {
      newUrl += "|COMPONENT=HLS";
    }

    // Werbung finden
    var advert = null;

    // Gerade den Channel gestartet => on_start_ad
    if (lastVersion == null) {
      if (eventData["on_start_ad"] && !Utils.isStringEmpty(eventData["on_start_ad"]["vast_url"])) {
        advert = {
          urls: [eventData["on_start_ad"]["vast_url"]],
          fallback_url: eventData["on_start_ad"]["vast_url_fallback"]
        }
      }
      if (eventData["ad"] && eventData["ad"]["version"]) {
        advertId = eventData["ad"]["version"];
      }
    } else {
      // ansonsten normale ad
      if (eventData["ad"] && eventData["ad"]["version"]) {
        newAdvertId = eventData["ad"]["version"];
        // wenn die Ad-ID neu ist, die benutzen
        if (advertId != null && advertId != newAdvertId) {
          advert = {
            urls: [eventData["ad"]["vast_url"]],
            fallback_url: eventData["ad"]["vast_url_fallback"]
          }
        }
        advertId = newAdvertId;
      }
    }

    // Curtain setzen wenn keine Werbung da ist
    if (advert == null) {
      // Curtain setzen
      if (eventData["curtain_picture"] && !Utils.isStringEmpty(eventData["curtain_picture"]["url"])) {
        if (eventData["curtain_picture"]["url"] != curtainPicture) {
          MessageQueue.sendMessage("hide_bauchbinde");
          curtainPicture = eventData["curtain_picture"]["url"];
          $("body").remove("#curtain_picture");
          $("#putpat_view").prepend("<div id='curtain_picture'><img src='"+curtainPicture+"'></div>");
        }
      } else {
        if (curtainPicture != null) {
          MessageQueue.sendMessage("show_bauchbinde");
        }
        curtainPicture = null;
        $("#curtain_picture").remove();
      }
    } else {
      curtainPicture = null;
      $("#curtain_picture").remove();
    }

    if (advert != null && !Utils.isStringEmpty(advert.urls[0])) {
      ChannelView.adClient.loadAdvertisement(advert, function(status, ad) {
        ChannelView.updatingClip = false;
        ChannelView.playAdvert(status, ad);
      });
    } else if (eventData["play_stream"]) {
      ChannelView.updateChannelShowPicture();
      MessageQueue.sendMessage("new_clip", {artist: ChannelView.currentClip["artist"], title: ChannelView.currentClip["title"]});
      if ((Player.url != newUrl) || (Player.getState() != Player.PLAYING)) {
        Player.stopVideo(false);
        Player.setVideoURL(newUrl);
        ChannelView.updatePlayerSize();
        Player.playVideo(0);
     }
    } else {
        Player.stopVideo(false);
    }
  }

  self.start = function(liveId) {
    clearInterval(bauchbindeTimer);
    Loader.start();
    lastVersion = null;
    advertId = null;
    ChannelView.submitCurrentClip(function() {
      ChannelView.updateChannelLogo();
      loadData(liveId);
    });
  };

  self.stop = function() {
    MessageQueue.sendMessage("enable_bauchbinde");
    $("#curtain_picture").remove();
    clearInterval(bauchbindeTimer);
  }

  self.currentClip = function() {
    return currentClip;
  }

  self.curtainVisible = function() {
    return curtainPicture != null;
  }

  return self;
}();
